# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import django
import django.test.utils
from django.conf import settings


settings.configure()

django.setup()
django.test.utils.setup_test_environment()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
from enum import Enum

from rest_framework.exceptions import ValidationError
from rest_framework.fields import ChoiceField
from rest_witchcraft.fields import EnumField, HyperlinkedIdentityField, ImplicitExpandableListField

from django.test import SimpleTestCase

from .models import Owner


class SomeEnum(Enum):
    test1 = 1
    test2 = 2


class TestEnumField(SimpleTestCase):
    def test_choices(self):

        field = EnumField(enum_class=SomeEnum)

        self.assertEqual(field.choices, {1: 1, 2: 2})

    def test_to_internal_value_works(self):

        field = EnumField(enum_class=SomeEnum)

        value = field.to_internal_value(2)

        self.assertEqual(value, SomeEnum.test2)

    def test_to_internal_value_validates(self):

        field = EnumField(enum_class=SomeEnum)

        with self.assertRaises(ValidationError):
            field.to_internal_value("test3")

    def test_to_representation_works(self):

        field = EnumField(enum_class=SomeEnum)

        value = field.to_representation(SomeEnum.test1)

        self.assertEqual(value, 1)

    def test_to_representation_handles_none(self):

        field = EnumField(enum_class=SomeEnum)

        value = field.to_representation(None)

        self.assertIsNone(value)


class TestHyperlinkedIdentityField(SimpleTestCase):
    def test_url_not_saved(self):
        field = HyperlinkedIdentityField(view_name="foo")

        self.assertIsNone(field.get_url(Owner(first_name="Jon", last_name="Snow"), None, None, None))


class TestImplicitExpandableListField(SimpleTestCase):
    def test_to_internal_value(self):
        f = ImplicitExpandableListField(child=ChoiceField(choices=["foo", "foo__bar", "bar"]))

        self.assertEqual(f.run_validation(["foo"]), ["foo"])
        self.assertEqual(set(f.run_validation(["foo__bar"])), {"foo__bar", "foo"})



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from rest_framework import generics

from sqlalchemy.exc import InvalidRequestError

from django.http import Http404

from django_sorcery.db.meta import model_info
from django_sorcery.db.models import get_primary_keys
from django_sorcery.utils import suppress


class GenericAPIView(generics.GenericAPIView):
    """
    Base class for sqlalchemy specific views
    """

    @classmethod
    def get_model(cls):
        """
        Returns the model class
        """
        model = None

        with suppress(AttributeError, InvalidRequestError):
            model = cls.queryset._only_entity_zero().class_

        if model:
            return model

        with suppress(AttributeError):
            model = cls.serializer_class.Meta.model

        assert model is not None, (
            "Couldn't figure out the model for {viewset} attribute, either provide a"
            "queryset or a serializer with a Meta.model".format(viewset=cls.__name__)
        )

        return model

    def get_session(self):
        """
        Returns the session
        """
        queryset = self.get_queryset()
        return queryset.session

    def get_object(self):
        """
        Returns the object the view is displaying.

        We ignore the `lookup_field` and `lookup_url_kwarg` values
        only when tere are multiple primary keys
        """
        queryset = self.get_queryset()
        model = self.get_model()
        info = model_info(model)
        kwargs = self.kwargs.copy()

        # we want to honor DRF lookup_field and lookup_url_kwarg API
        # but only if they are defined and there is single primary key.
        # When there are multiple, all bets are off so we restrict url kwargs
        # to model column names
        if len(info.primary_keys) == 1:
            lookup_field = self.lookup_field
            lookup_url_kwarg = self.lookup_url_kwarg or lookup_field

            kwargs[lookup_field] = kwargs.pop(lookup_url_kwarg)

        obj = queryset.get(get_primary_keys(model, kwargs))

        if not obj:
            raise Http404("No %s matches the given query." % model.__name__)

        return obj



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from rest_framework import mixins, viewsets

from .generics import GenericAPIView
from .mixins import DestroyModelMixin, ExpandableQuerySerializerMixin


class GenericViewSet(viewsets.ViewSetMixin, GenericAPIView):
    """
    The GenericViewSet class does not provide any actions by default, but does include the base set of generic view
    behavior, such as the `get_object` and `get_queryset` methods.
    """


class ReadOnlyViewModelViewSet(mixins.RetrieveModelMixin, mixins.ListModelMixin, GenericViewSet):
    """
    A viewset that provides default `list()` and `retrieve()` actions.
    """


class ModelViewSet(
    mixins.CreateModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    DestroyModelMixin,
    mixins.ListModelMixin,
    GenericViewSet,
):
    """
    A viewset that provides default `create()`, `retrieve()`, `update()`, `partial_update()`, `destroy()` and `list()`
    actions.
    """


class ExpandableModelViewSet(ExpandableQuerySerializerMixin, ModelViewSet):
    """
    A viewset that provides automatically eagerloadsany subfields that are expanded via querystring.

    For queryset to be expanded, either :py:class:`rest_witchcraft.serializers.ExpandableModelSerializer`
    needs to be used in ``serializer_class`` or ``query_serializer_class`` can be manually provided.
    """



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from rest_framework.serializers import ValidationError
from rest_framework.settings import api_settings

from django.core.exceptions import NON_FIELD_ERRORS, ValidationError as DjangoValidationError


def _django_to_drf(e):
    if hasattr(e, "error_dict") or isinstance(e, dict):
        return {
            k if not k == NON_FIELD_ERRORS else api_settings.NON_FIELD_ERRORS_KEY: _django_to_drf(v)
            for k, v in getattr(e, "error_dict", e).items()
        }
    elif hasattr(e, "error_list"):
        return e.messages
    elif isinstance(e, list):
        errors = []
        for j in e:
            if isinstance(j, DjangoValidationError):
                errors += _django_to_drf(j)
            else:
                errors.append(_django_to_drf(j))
        return errors
    return e


def django_to_drf_validation_error(e):
    return ValidationError(
        _django_to_drf(e) if hasattr(e, "error_dict") else {api_settings.NON_FIELD_ERRORS_KEY: e.messages}
    )
